@@ -1,105 +1,49 @@
-#define _GNU_SOURCE
+/*
+  Copyright (C) 2008 Ben Asselstine
+  Written by Ben Asselstine
+
+  This file is part of fileschanged.
+
+  fileschanged is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
+
+  fileschanged is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with fileschanged; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301  USA
+*/
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <error.h>
-#include "gettext.h"
 #include <libgen.h>
 #include <unistd.h>
 #include "filelist.h"
-#include "xrealpath.h"
+#include <argz.h>
 #include "node.h"
 #include "list.h"
-#include "opts.h"
 #include "listdirs.h"
-extern struct arguments_t arguments;
 
-/*
- * filelist.c:
- *   int filelist_populate (void *list_of_files_to_monitor);
- *
- * Fill LIST_OF_FILES_TO_MONITOR which is an instantianted list from list.[ch], 
- * with entries that are node_t from node.[ch].  Each entry represents a file
- * that we want to monitor.  The source data for this list is either from 
- * files specified on the command-line, or from a filelist specified by the -l
- * option.
- *
- * Returns 0 upon success, or non-zero upon error.
- *
- */
-static int 
-for_every_filename (int (*for_every_file)(int (*)(void *, char *), void *list), int (*add_it_to_the)(void *list, char *filename), void *list)
-{
-  for_every_file (add_it_to_the, list);
-  return 0;
-}
 
 static int 
-on_the_command_line (int (*process_file)(void *list, char *filename), void *list)
-{
-  unsigned int i;
-  int retval;
-  for(i = 0; i < arguments.arraylen; i++)
-    {
-      retval = process_file (list, arguments.args[i]);
-    }
-  return 0;
-}
-
-static void 
-chop (char *line)
-{
-  char *tmp;
-  tmp = strpbrk (line, "\r\n");
-  if (tmp)
-    tmp[0] = '\0';
-  return;
-}
-
-static int 
-in_filelist_file (int (*process_file)(void *list, char *filename), void *list)
-{
-  //read the list of files from a file.
-  FILE *fileptr;
-  char *filename;
-  char *line = NULL;
-  size_t n = 0;
-  int retval;
-  filename = arguments.fileschanged.filelist_filename;
-  if (strcmp (filename, "-") == 0)
-    fileptr = stdin;
-  else
-    {
-      fileptr = fopen (filename, "r");
-      if (!fileptr)
-	error(1, 0, _("Error: couldn't open '%s'\n"), filename);
-    }
-  while (getline (&line, &n, fileptr) > 0)
-    {
-      chop (line);
-      if (line[0] != '#')
-	retval = process_file (list, line);
-      free (line); line = NULL;
-      n = 0;
-    }
-
-  if (fileptr != stdin)
-    fclose (fileptr);
-  return 0;
-}
-
-static int 
-add_it_to_the (void *list, char *filename)
+process_file (void *list, char *filename, int recurse, int deref_symlinks)
 {
   int retval;
   struct node_t node;
-  retval = node_new (&node, filename);
+  retval = node_new (&node, filename, deref_symlinks);
   if (retval == 0)
     {
       if ((S_ISDIR (node.statbuf.st_mode)) && 
-	  (arguments.fileschanged.recursive))
+	  (recurse))
 	{
 	  void *dirs = NULL;
 	  struct node_t *dir = NULL;
@@ -108,7 +52,7 @@ add_it_to_the (void *list, char *filename)
 	  //add all dirs below and including this one.
 
 	  list_init (&dirs);
-	  retval = listdirs (&node, dirs);
+	  retval = listdirs (&node, dirs, deref_symlinks);
 	  if (retval != 0)
 	    return -1;
 	  list_count (dirs, &count);
@@ -179,18 +123,30 @@ remove_duplicates (void *list)
   return 0;
 }
 
+/*
+ * filelist.c:
+ *   int filelist_populate (void *list_of_files_to_monitor, char *filelist_argz,
+ *   char *filelist_argz_len, int recurse);
+ *
+ * Fill LIST_OF_FILES_TO_MONITOR which is an instantianted list from list.[ch], 
+ * with entries that are node_t from node.[ch].  Each entry represents a file
+ * that we want to monitor.  The source data for this list is from the
+ * argz & argz_len parameters.  If recurse is set, then directories are
+ * descended into.
+ *
+ * Returns 0 upon success, or non-zero upon error.
+ *
+ */
 int 
-filelist_populate (void *list_of_files_to_monitor)
+filelist_populate (void *list_of_files_to_monitor, char *filelist_argz, 
+		   size_t filelist_argz_len, int recurse, int deref_symlinks)
 {
   int retval = 0;
-  if (arguments.fileschanged.filelist)
-    retval = for_every_filename (in_filelist_file, add_it_to_the, 
-				 list_of_files_to_monitor);
-  else if (arguments.fileschanged.filestomonitor)
-    retval = for_every_filename (on_the_command_line, add_it_to_the, 
-				 list_of_files_to_monitor);
-  else
-    return -1;
+  char *entry = NULL;
+  while ((entry = argz_next (filelist_argz, filelist_argz_len, entry)))
+    {
+      process_file (list_of_files_to_monitor, entry, recurse, deref_symlinks);
+    }
   //okay we have our list now.
   //now remove files for which there are directories.
 
