@@ -1,10 +1,27 @@
+/*
+  Copyright (C) 2008 Ben Asselstine
+  Written by Ben Asselstine
+
+  This file is part of fileschanged.
+
+  fileschanged is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
+
+  fileschanged is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with fileschanged; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301  USA
+*/
 #include <string.h>
 #include <stdlib.h>
 #include "node.h"
-#include "xrealpath.h"
-#include "opts.h"
-
-extern struct arguments_t arguments;
+#include "canonicalize.h"
 
 void 
 node_free(struct node_t *node)
@@ -24,20 +41,20 @@ node_copy(struct node_t *dst, struct node_t *src)
 }
 
 int 
-node_new (struct node_t *node, char *filename)
+node_new (struct node_t *node, char *filename, int deref_symlinks)
 {
   int retval;
   if ((!node) || (!filename))
     return -1;
   memset (node, 0, sizeof (struct node_t));
-  if (arguments.fileschanged.dereference_symlinks)
+  if (deref_symlinks)
     retval = stat (filename, &node->statbuf);
   else
     retval = lstat (filename, &node->statbuf);
   if (retval != 0)
     return -2;
   if (!S_ISLNK (node->statbuf.st_mode))
-    node->filename = xrealpath (filename);
+    node->filename = canonicalize_file_name (filename);
   else
     node->filename = strdup (filename);
   if (!node->filename)
